<?php
/**
 * File: Plugwise.class.php
 * User: Kevin B
 * Date: 01-11-12
 * Time: 19:50
 */
class Plugwise {

    /**
     * @var null
     */
    private $_host = NULL;

    /**
     * @var null
     */
    private $_port = NULL;

    /**
     * @var null
     */
    private $_username = NULL;

    /**
     * @var null
     */
    private $_password = NULL;

    /**
     * @var int
     */
    private $_timeOut = 0;

    /**
     * @var array
     */
    private $_data = NULL;

    /**
     * @var string
     */
    private $_endpointData = '/statistics.xml';

    /**
     * @var string
     */
    private $_endpointActions = '/api/actions.html';

    /**
     * @param string $host
     * @param int $port
     * @param int $timeOut
     * @param string $username
     * @param string $password
     */
    public function __construct($host, $port, $timeOut, $username=NULL, $password=NULL) {
        $this->_host        = $host;
        $this->_port        = $port;
        $this->_timeOut     = $timeOut;
        $this->_username    = $username;
        $this->_password    = $password;

        // Set socket timeout
        ini_set('default_socket_timeout', $timeOut);
    }

    /**
     * @return void
     */
    private function getData() {

        // Prepare data URL
        $dataUrl = 'http://'.$this->_host.':'.$this->_port.$this->_endpointData;

        // Get data
        if($data = $this->httpRequest($dataUrl)) {
            // Get process XML from data
            $this->_data = simplexml_load_string($data);
        }
    }

    /**
     * @return array
     */
    public function getAllAppliances() {
        $return = false;
        // If there is no device data
        if(is_null($this->_data)) {
            // Get device data
            $this->getData();
        }

        // If there are appliances found
        if(isset($this->_data->appliance)) {
            // Loop through appliances
            foreach($this->_data->appliance as $appliance) {
                // Determine address
                $address = (string)strtoupper($appliance->macaddr);
                $address = substr($address, -6, 6);

                // Build appliance data array
                $return[$address] = array(
                    'id'            => (integer)$appliance->id,
                    'name'          => (string)trim($appliance->name),
                    'type'          => (string)trim($appliance->type),
                    'image'         => (string)trim($appliance->image),
                    'locked'        => ((string)trim($appliance->locked) == 'True' ? true : false),
                    'realstate'     => ((string)trim($appliance->realstate) == 'on' ? 1 : 0),
                    'powerstate'    => ((string)trim($appliance->powerstate) == 'on' ? 1 : 0),
                    'powerusage'    => (float)($appliance->powerusage / 100),
                    'totalusage'    => (float)($appliance->totalusage / 100),
                    'moduleid'      => (integer)$appliance->moduleid,
                    'macaddr'       => (string)strtoupper($appliance->macaddr),
                    'moduletype'    => (string)trim($appliance->moduletype),
                    'room'          => (string)trim($appliance->room)
                );
            }
        }

        return $return;
    }

    /**
     * @param string $address
     * @return array
     */
    public function getAppliance($address) {
        $return = false;
        $address = trim($address);

        // Get all appliances
        $appliances = $this->getAllAppliances();

        // Check if appliances exists
        if(isset($appliances[$address])) {
            $return = $appliances[$address];
        }

        return $return;
    }

    /**
     * @return int $return
     */
    public function getSourceTimeStamp() {
        $return = false;
        // If there is no device data
        if(is_null($this->_data)) {
            // Get device data
            $this->getData();
        }

        // If source date is available
        if(isset($this->_data->source->datetime)) {
            $return = strtotime(trim($this->_data->source->datetime));
        }

        return $return;
    }

    /**
     * @return string $return
     */
    public function getSourceVersion() {
        $return = false;
        // If there is no device data
        if(is_null($this->_data)) {
            // Get device data
            $this->getData();
        }

        // If source version is available
        if(isset($this->_data->source->version)) {
            $return = trim($this->_data->source->version);
        }

        return $return;
    }

    /**
     * @return string $return
     */
    public function getSourceXmlVersion() {
        $return = false;
        // If there is no device data
        if(is_null($this->_data)) {
            // Get device data
            $this->getData();
        }

        // If source XML version is available
        if(isset($this->_data->source->versionxml)) {
            $return = trim($this->_data->source->versionxml);
        }

        return $return;
    }

    /**
     * @return string $return
     */
    public function getSourceLanguage() {
        $return = false;
        // If there is no device data
        if(is_null($this->_data)) {
            // Get device data
            $this->getData();
        }

        // If source language is available
        if(isset($this->_data->source->language)) {
            $return = trim($this->_data->source->language);
        }

        return $return;
    }

    /**
     * @param string $address
     * @return bool $return
     */
    public function switchOn($address) {
        $return = false;

        // If there is no device data
        if(is_null($this->_data)) {
            // Get device data
            $this->getData();
        }

        // Get appliance data
        if($appliance = $this->getAppliance($address)) {
            // Do switch action
            $return = $this->doAction($appliance['id'], 'switchon');
        }

        return $return;
    }

    /**
     * @param string $address
     * @return bool $return
     */
    public function switchOff($address) {
        $return = false;

        // If there is no device data
        if(is_null($this->_data)) {
            // Get device data
            $this->getData();
        }

        // Get appliance data
        if($appliance = $this->getAppliance($address)) {
            // Do switch action
            $return = $this->doAction($appliance['id'], 'switchoff');
        }

        return $return;
    }

    /**
     * @param string $address
     * @return bool $return
     */
    public function lockOn($address) {
        $return = false;

        // If there is no device data
        if(is_null($this->_data)) {
            // Get device data
            $this->getData();
        }

        // Get appliance data
        if($appliance = $this->getAppliance($address)) {
            // Do lock action
            $return = $this->doAction($appliance['id'], 'lockon');
        }

        return $return;
    }

    /**
     * @param string $address
     * @return bool $return
     */
    public function lockOff($address) {
        $return = false;

        // If there is no device data
        if(is_null($this->_data)) {
            // Get device data
            $this->getData();
        }

        // Get appliance data
        if($appliance = $this->getAppliance($address)) {
            // Do lock action
            $return = $this->doAction($appliance['id'], 'lockoff');
        }

        return $return;
    }

    /**
     * @param int $applianceId
     * @param string $option
     * @return bool $return
     */
    private function doAction($applianceId, $option) {
        $return = false;
        $applianceId = (integer)$applianceId;
        $option = trim($option);
        $option = strtolower($option);

        // Build action url
        $actionUrl = 'http://'.$this->_host.':'.$this->_port.$this->_endpointActions.'?id='.$applianceId.'&option='.$option;

        // Call action url
        if($response = $this->httpRequest($actionUrl)) {
            // Process response
            $response = substr($response, 2);
            $response = substr($response, 0, -1);
            $response = trim($response);
            $response = str_replace("'", '', $response);
            $response = str_replace(' ', '', $response);
            $fields = explode(',', $response);
            foreach($fields as $field) {
                $values = explode('=>', $field);
                $reponseData[trim($values[0])] = trim($values[1]);
            }

            if($reponseData['id'] == $applianceId && $reponseData['option'] == $option) {
                $return = true;
            }
        }

        return $return;
    }

    /**
     * @param $url
     * @return bool|string $return
     */
    private function httpRequest($url) {
        $url = trim($url);
        $return = false;

        $streamContext = NULL;
        // Needs HTTP autentication?
        if(!is_null($this->_username) && !is_null($this->_password)) {
            $username = trim($this->_username);
            $password = trim($this->_password);

            $context['http'] = array(
                'header'  => "Authorization: Basic ".base64_encode("$username:$password")
            );

            $streamContext = stream_context_create($context);
        }

        if($data = file_get_contents($url, false, $streamContext)) {
            $return = $data;
        }

        return $return;
    }
}
?>